\name{MakeGridFromCoords}
\alias{MakeGridFromCoords}
\title{
Create a Data Frame of Cell Boundaries From Vectors of Coordinates
}
\description{
A function to create a data frame of cell boundary values from vectors of \emph{x} and \emph{y} coordinates.  This function is commonly used to create inputs suitable for entry into the \code{LatticeTransitionProbs} function.
}
\usage{
MakeGridFromCoords(x, y)
}
\arguments{
  \item{x}{
A double vector denoting the locations of cell boundaries of a lattice on the \emph{x}-axis.  \code{range(x)} defines the total extent of the lattice on the \emph{x}-axis.
}
  \item{y}{
A double vector denoting the locations of cell boundaries of a lattice on the \emph{y}-axis.  \code{range(y)} defines the total extent of the lattice on the \emph{y}-axis.
}
}
\value{
A data frame with \code{(length(x.coords) - 1) * (length(y.coords) - 1)} rows containing the boundary coordinates for each cell in the lattice:
  \item{x1}{
Lower \emph{x}-coordinate of each cell in the lattice
}
  \item{x2}{
Upper \emph{x}-coordinate of each cell in the lattice
}
  \item{y1}{
Lower \emph{y}-coordinate of each cell in the lattice
}
  \item{y2}{
Upper \emph{y}-coordinate of each cell in the lattice
}
}
\author{
Joseph Chipperfield <\email{joechip90@googlemail.com}>
}
\seealso{
\code{\link{LatticeTransitionProbs}}
}
\examples{
# Create a data frame of cell boundary coordinates of cell of a 5x5
# lattice with regular cell length of one unit
x.coords <- seq(0.0, 5.0, 1.0)
y.coords <- seq(0.0, 5.0, 1.0)
MakeGridFromCoords(x.coords, y.coords)
}
\keyword{manip}
