#ifndef __EXTSTRING_H__
#define __EXTSTRING_H__

#include "ExtNumber.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{	// -----------------------
	//  EXTENDED STRING CLASS
	// -----------------------
	// Class that adds features to the standard string class
	class ExtString
	{
	private:
		char *pString;			// Array of characters to hold string
		size_t ulStringSize;	// Size of the character array
	public:
		// Functions for null initialisation
		ExtString() throw() : pString(NULL), ulStringSize(0) {};
		void Initialise() throw();
		// Parameterised initialisation functions
		ExtString(const char *pStringIn, size_t ulSizeIn) throw(Exception) : pString(NULL), ulStringSize(0) {Initialise(pStringIn, ulSizeIn);};
		void Initialise(const char *pStringIn, size_t ulSizeIn) throw(Exception);
		ExtString(const char *pStringIn) throw(Exception) : pString(NULL), ulStringSize(0) {Initialise(pStringIn);};
		void Initialise(const char *pStringIn) throw(Exception);
		ExtString& operator = (const char *pStringIn) throw(Exception) {Initialise(pStringIn); return *this;};
		// Copy initialisation functions
		ExtString(const ExtString &cObIn) throw(Exception) : pString(NULL), ulStringSize(0) {Initialise(cObIn);};
		void Initialise(const ExtString &cObIn) throw(Exception);
		ExtString& operator = (const ExtString &cObIn) throw(Exception) {Initialise(cObIn); return *this;};
		// Special value initialisation functions
		ExtString(SpecialValues eSpecIn) throw(Exception) : pString(NULL), ulStringSize(0) {Initialise(eSpecIn);};
		void Initialise(SpecialValues eSpecIn) throw(Exception);
		ExtString& operator = (SpecialValues eSpecIn) throw(Exception) {Initialise(eSpecIn); return *this;};
		// Destructor function
		~ExtString() throw() {Initialise();};
		// Tests of equality
		ExtBool Equal(const ExtString &cObIn) const throw();
		ExtBool NotEqual(const ExtString &cObIn) const throw() {return Equal(cObIn).Inverse();};
		ExtBool operator == (const ExtString &cObIn) const throw() {return Equal(cObIn);};
		ExtBool operator != (const ExtString &cObIn) const throw() {return NotEqual(cObIn);};
		ExtBool Equal(const char *pStringIn) const throw();
		ExtBool NotEqual(const char *pStringIn) const throw() {return Equal(pStringIn).Inverse();};
		ExtBool operator == (const char *pStringIn) const throw() {return Equal(pStringIn);};
		ExtBool operator != (const char *pStringIn) const throw() {return NotEqual(pStringIn);};
		// Tests of string state
		bool IsMissingValue() const throw() {return pString == NULL;};		// Returns true if string is empty
		// Returns the string
		char *RetrieveCString(char *pStringOut, size_t ulSize) const throw();
		// Return the length of the string
		size_t Length() const throw() {return ulStringSize;};
	};

	// Other overloaded equality operators to interact with native C-string
	inline ExtBool operator == (const char *pStringIn, const ExtString &cObIn) throw() {return cObIn.Equal(pStringIn);}
	inline ExtBool operator != (const char *pStringIn, const ExtString &cObIn) throw() {return cObIn.NotEqual(pStringIn);}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
