// LatticeDispersal.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "LatticeDispersal.h"

#ifdef _MSC_VER

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

#endif

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace
{	// Size of character buffer used in title calculation
	const size_t LD_STRBUFF_SIZE = 255;
	
	// Function to calculate centroid-to-centroid Gaussian transition probabilities (unnormalised)
	double CCGaussTrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2, double dAlpha) throw()
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		dOut = exp(-StdExtensions::Sqrd((dMaxKX + dMinKX - dMaxJX - dMinJX) / (2.0 * dAlpha)))
			* exp(-StdExtensions::Sqrd((dMaxKY + dMinKY - dMaxJY - dMinJY) / (2.0 * dAlpha)));
		return dOut;
	}

	// Function to calculate centroid-to-centroid transition probabilities (unnormalised)
	double CCTrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2,
		RInterface::RFunction cWrap, const RInterface::RList &cExtraParams) throw(StdExtensions::Exception)
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		RInterface::RList cParams;
		RInterface::RDoubleVector cTemp;
		dOut = 0.0;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		try
		{	// Set all the function input parameters
			cParams.Initialise(cExtraParams);
			cTemp.Initialise(1);
			cTemp.GetElement(0).Initialise(dMinJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x1");
			cTemp.GetElement(0).Initialise(dMaxJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x2");
			cTemp.GetElement(0).Initialise(dMinJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y1");
			cTemp.GetElement(0).Initialise(dMaxJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y2");
			cTemp.GetElement(0).Initialise(dMinKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x1");
			cTemp.GetElement(0).Initialise(dMaxKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x2");
			cTemp.GetElement(0).Initialise(dMinKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y1");
			cTemp.GetElement(0).Initialise(dMaxKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y2");
			// Call the function
			cTemp.ImportFromSEXP(RInterface::CallRFunction(cWrap, cParams));
			dOut = cTemp.GetElement(0).GetAsDouble();
		}
		// Catch any exceptions thrown
		catch(StdExtensions::Exception cXa)
		{
			throw cXa;
		}
		catch(std::bad_cast)
		{
			StdExtensions::RaiseException("casting error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{
			StdExtensions::RaiseException("unknown error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_UnknownError);
		}
		return dOut;
	}

	// Function to calculate centroid-to-area Gaussian transition probabilities
	double CAGaussTrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2, double dAlpha) throw(StdExtensions::Exception)
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		if(dMinKX == dMaxKX || dMinKY == dMaxKY)
		{
			StdExtensions::RaiseException("destination cell has no area", __LINE__, "CAGaussTrans(double, double, double, double, double, double, double, double, double)", __FILE__, StdExtensions::e_DomainError);
		}
		dOut = 0.25 * (StdExtensions::Erf((2.0 * dMaxKX - dMaxJX - dMinJX) / (2.0 * dAlpha)) - StdExtensions::Erf((2.0 * dMinKX - dMaxJX - dMinJX) / (2.0 * dAlpha)))
			* (StdExtensions::Erf((2.0 * dMaxKY - dMaxJY - dMinJY) / (2.0 * dAlpha)) - StdExtensions::Erf((2.0 * dMinKY - dMaxJY - dMinJY) / (2.0 * dAlpha)));
		return dOut;
	}

	// Function to calculate centroid-to-area transition probabilities (unnormalised)
	double CATrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2,
		RInterface::RFunction cWrap, const RInterface::RList &cExtraParams) throw(StdExtensions::Exception)
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		RInterface::RList cParams;
		RInterface::RDoubleVector cTemp;
		dOut = 0.0;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		try
		{	// Set all the function input parameters
			cParams.Initialise(cExtraParams);
			cTemp.Initialise(1);
			cTemp.GetElement(0).Initialise(dMinJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x1");
			cTemp.GetElement(0).Initialise(dMaxJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x2");
			cTemp.GetElement(0).Initialise(dMinJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y1");
			cTemp.GetElement(0).Initialise(dMaxJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y2");
			cTemp.GetElement(0).Initialise(dMinKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x1");
			cTemp.GetElement(0).Initialise(dMaxKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x2");
			cTemp.GetElement(0).Initialise(dMinKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y1");
			cTemp.GetElement(0).Initialise(dMaxKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y2");
			// Call the function
			cTemp.ImportFromSEXP(RInterface::CallRFunction(cWrap, cParams));
			dOut = cTemp.GetElement(0).GetAsDouble();
		}
		// Catch any exceptions thrown
		catch(StdExtensions::Exception cXa)
		{
			throw cXa;
		}
		catch(std::bad_cast)
		{
			StdExtensions::RaiseException("casting error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{
			StdExtensions::RaiseException("unknown error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_UnknownError);
		}
		return dOut;
	}

	// Function to calculate area-to-centroid Gaussian transition probabilities (unnormalised)
	double ACGaussTrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2, double dAlpha) throw(StdExtensions::Exception)
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		if(dMinJX == dMaxJX || dMinJY == dMaxJY)
		{
			StdExtensions::RaiseException("source cell has no area", __LINE__, "ACGaussTrans(double, double, double, double, double, double, double, double, double)", __FILE__, StdExtensions::e_DomainError);
		}
		dOut = (StdExtensions::Erf((dMaxKX + dMinKX - 2.0 * dMaxJX) / (2.0 * dAlpha)) - StdExtensions::Erf((dMaxKX + dMinKX - 2.0 * dMinJX) / (2.0 * dAlpha)))
			* (StdExtensions::Erf((dMaxKY + dMinKY - 2.0 * dMaxJY) / (2.0 * dAlpha)) - StdExtensions::Erf((dMaxKY + dMinKY - 2.0 * dMinJY) / (2.0 * dAlpha)));
		return dOut;
	}

	// Function to calculate area-to-centroid transition probabilities (unnormalised)
	double ACTrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2,
		RInterface::RFunction cWrap, const RInterface::RList &cExtraParams) throw(StdExtensions::Exception)
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		RInterface::RList cParams;
		RInterface::RDoubleVector cTemp;
		dOut = 0.0;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		try
		{	// Set all the function input parameters
			cParams.Initialise(cExtraParams);
			cTemp.Initialise(1);
			cTemp.GetElement(0).Initialise(dMinJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x1");
			cTemp.GetElement(0).Initialise(dMaxJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x2");
			cTemp.GetElement(0).Initialise(dMinJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y1");
			cTemp.GetElement(0).Initialise(dMaxJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y2");
			cTemp.GetElement(0).Initialise(dMinKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x1");
			cTemp.GetElement(0).Initialise(dMaxKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x2");
			cTemp.GetElement(0).Initialise(dMinKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y1");
			cTemp.GetElement(0).Initialise(dMaxKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y2");
			// Call the function
			cTemp.ImportFromSEXP(RInterface::CallRFunction(cWrap, cParams));
			dOut = cTemp.GetElement(0).GetAsDouble() / ((dMaxJX - dMinJX) * (dMaxJY - dMinJY));
		}
		// Catch any exceptions thrown
		catch(StdExtensions::Exception cXa)
		{
			throw cXa;
		}
		catch(std::bad_cast)
		{
			StdExtensions::RaiseException("casting error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{
			StdExtensions::RaiseException("unknown error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_UnknownError);
		}
		return dOut;
	}

	// Function to calculate area-to-area Gaussian transition probabilities
	double AAGaussTrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2, double dAlpha) throw(StdExtensions::Exception)
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		if(dMinKX == dMaxKX || dMinKY == dMaxKY)
		{
			StdExtensions::RaiseException("destination cell has no area", __LINE__, "AAGaussTrans(double, double, double, double, double, double, double, double, double)", __FILE__, StdExtensions::e_DomainError);
		}
		if(dMinJX == dMaxJX || dMinJY == dMaxJY)
		{
			StdExtensions::RaiseException("source cell has no area", __LINE__, "AAGaussTrans(double, double, double, double, double, double, double, double, double)", __FILE__, StdExtensions::e_DomainError);
		}
		dOut = (1.0 / ((dMaxJX - dMinJX) * (dMaxJY - dMinJY) * 4.0)) *
			(	(dMaxKY - dMaxJY) * StdExtensions::Erf((dMaxKY - dMaxJY) / dAlpha) - (dMinKY - dMaxJY) * StdExtensions::Erf((dMinKY - dMaxJY) / dAlpha)
				- (dMaxKY - dMinJY) * StdExtensions::Erf((dMaxKY - dMinJY) / dAlpha) + (dMinKY - dMinJY) * StdExtensions::Erf((dMinKY - dMinJY) / dAlpha)
				+ (dAlpha / sqrt(StdExtensions::EXT_PI)) * (exp(-StdExtensions::Sqrd((dMaxKY - dMaxJY) / dAlpha)) - 
				exp(-StdExtensions::Sqrd((dMinKY - dMaxJY) / dAlpha)) - exp(-StdExtensions::Sqrd((dMaxKY - dMinJY) / dAlpha))
				+ exp(-StdExtensions::Sqrd((dMinKY - dMinJY) / dAlpha)))) *
			(	(dMaxKX - dMaxJX) * StdExtensions::Erf((dMaxKX - dMaxJX) / dAlpha) - (dMinKX - dMaxJX) * StdExtensions::Erf((dMinKX - dMaxJX) / dAlpha)
				- (dMaxKX - dMinJX) * StdExtensions::Erf((dMaxKX - dMinJX) / dAlpha) + (dMinKX - dMinJX) * StdExtensions::Erf((dMinKX - dMinJX) / dAlpha)
				+ (dAlpha / sqrt(StdExtensions::EXT_PI)) * (exp(-StdExtensions::Sqrd((dMaxKX - dMaxJX) / dAlpha)) - 
				exp(-StdExtensions::Sqrd((dMinKX - dMaxJX) / dAlpha)) - exp(-StdExtensions::Sqrd((dMaxKX - dMinJX) / dAlpha))
				+ exp(-StdExtensions::Sqrd((dMinKX - dMinJX) / dAlpha))));
		return dOut;
	}

	// Function to calculate area-to-area transition probabilities (unnormalised)
	double AATrans(double dJX1, double dJX2, double dJY1, double dJY2,
		double dKX1, double dKX2, double dKY1, double dKY2,
		RInterface::RFunction cWrap, const RInterface::RList &cExtraParams) throw(StdExtensions::Exception)
	{	// Calculate minimum and maximum extents of the source and destination cells
		double dMaxJX, dMinJX, dMaxJY, dMinJY, dMaxKX, dMinKX, dMaxKY, dMinKY, dOut;
		RInterface::RList cParams;
		RInterface::RDoubleVector cTemp;
		dOut = 0.0;
		dMaxJX = dJX1 > dJX2 ? dJX1 : dJX2;
		dMinJX = dJX1 < dJX2 ? dJX1 : dJX2;
		dMaxJY = dJY1 > dJY2 ? dJY1 : dJY2;
		dMinJY = dJY1 < dJY2 ? dJY1 : dJY2;
		dMaxKX = dKX1 > dKX2 ? dKX1 : dKX2;
		dMinKX = dKX1 < dKX2 ? dKX1 : dKX2;
		dMaxKY = dKY1 > dKY2 ? dKY1 : dKY2;
		dMinKY = dKY1 < dKY2 ? dKY1 : dKY2;
		try
		{	// Set all the function input parameters
			cParams.Initialise(cExtraParams);
			cTemp.Initialise(1);
			cTemp.GetElement(0).Initialise(dMinJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x1");
			cTemp.GetElement(0).Initialise(dMaxJX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.x2");
			cTemp.GetElement(0).Initialise(dMinJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y1");
			cTemp.GetElement(0).Initialise(dMaxJY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "from.y2");
			cTemp.GetElement(0).Initialise(dMinKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x1");
			cTemp.GetElement(0).Initialise(dMaxKX);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.x2");
			cTemp.GetElement(0).Initialise(dMinKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y1");
			cTemp.GetElement(0).Initialise(dMaxKY);
			cParams.AddElementToFront(dynamic_cast<RInterface::RObject &>(cTemp), "to.y2");
			// Call the function
			cTemp.ImportFromSEXP(RInterface::CallRFunction(cWrap, cParams));
			dOut = cTemp.GetElement(0).GetAsDouble() / ((dMaxJX - dMinJX) * (dMaxJY - dMinJY));
		}
		// Catch any exceptions thrown
		catch(StdExtensions::Exception cXa)
		{
			throw cXa;
		}
		catch(std::bad_cast)
		{
			StdExtensions::RaiseException("casting error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{
			StdExtensions::RaiseException("unknown error thrown during probability calculation", __LINE__, "CCTrans(double, double, double, double, double, double, double, double, const RInterface::RList &)", __FILE__, StdExtensions::e_UnknownError);
		}
		return dOut;
	}

	// Function to check the stopping condition in periodic dispersal estimation
	bool CheckRunTot(double dRunTot, double dOldRunTot, double dMaxProb, LatticeDispersal::ApproxMethod eMethod) throw(StdExtensions::Exception)
	{
		bool bOut = true;
		switch(eMethod)
		{
		case LatticeDispersal::e_CC:
		case LatticeDispersal::e_AC:
			bOut = fabs(dRunTot - dOldRunTot) >= dMaxProb;
			break;
		case LatticeDispersal::e_CA:
		case LatticeDispersal::e_AA:
			bOut = dRunTot <= 1.0 - dMaxProb;
			break;
		default:
			StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "CheckRunTot(double, double, double, LatticeDispersal::ApproxMethod)", __FILE__, StdExtensions::e_DomainError);
		}
		return bOut;
	}
}

// Function to create a set of grid cells based on a set of x and y coordinates
//
//	x					-	Vector of x-coordinates of the grid cells
//	y					-	Vector of y-coordinates of the grid cells
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP MakeGridFromCoords(SEXP pArgs)
#else
	extern "C" SEXP MakeGridFromCoords(SEXP pArgs)
#endif
{
	RInterface::RList cOut;
	RInterface::RDoubleVector cX1, cX2, cY1, cY2, cXIn, cYIn;
	int iXSize, iYSize, iXIter, iYIter;
	iXSize = iYSize = iXIter = iYIter = 0;
	SEXP pOut = R_NilValue;
	try
	{	// Import the input vectors
		cXIn.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "x"));
		cYIn.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "y"));
		// Calculate the number of cells in each dimension
		iXSize = int(cXIn.Length());
		iYSize = int(cYIn.Length());
		// Size the outputs
		cX1.Initialise(size_t((iXSize - 1) * (iYSize - 1)));
		cX2.Initialise(size_t((iXSize - 1) * (iYSize - 1)));
		cY1.Initialise(size_t((iXSize - 1) * (iYSize - 1)));
		cY2.Initialise(size_t((iXSize - 1) * (iYSize - 1)));
		// Iterate over the different input cell coordinates
		for(iXIter = 0; iXIter < iXSize - 1; ++iXIter)
		{
			for(iYIter = 0; iYIter < iYSize - 1; ++iYIter)
			{	// Set the elements of the cell dimensions
				cX1.GetElement(iXIter * (iXSize - 1) + iYIter).Initialise(cXIn.GetElement(iXIter));
				cX2.GetElement(iXIter * (iXSize - 1) + iYIter).Initialise(cXIn.GetElement(iXIter + 1));
				cY1.GetElement(iXIter * (iXSize - 1) + iYIter).Initialise(cYIn.GetElement(iYIter));
				cY2.GetElement(iXIter * (iXSize - 1) + iYIter).Initialise(cYIn.GetElement(iYIter + 1));
			}
		}
		// Add the elements to the list
		cOut.AddElementToBack(dynamic_cast<RInterface::RObject &>(cX1), "x1");
		cOut.AddElementToBack(dynamic_cast<RInterface::RObject &>(cX2), "x2");
		cOut.AddElementToBack(dynamic_cast<RInterface::RObject &>(cY1), "y1");
		cOut.AddElementToBack(dynamic_cast<RInterface::RObject &>(cY2), "y2");
		// Export the output
		pOut = cOut.ExportAsSEXP();
	}
	catch(StdExtensions::Exception cXa)
	{
		// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(std::bad_cast)
	{
		// Write error if casting exception is thrown
		Rf_error("error thrown during pointer casting process");
	}
	catch(...)
	{
		// Write error if a non-standard exception is thrown
		Rf_error("unknown error");
	}
	return pOut;
}

// Function to evaluate the error function
//
//	x					-	Vector of doubles at which the error function evaluation is required
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP Erf(SEXP pArgs)
#else
	extern "C" SEXP Erf(SEXP pArgs)
#endif
{
	RInterface::RDoubleVector cIn, cOut;
	int iIter, iSize;
	SEXP pOut = R_NilValue;
	try
	{	// Import the input vector
		cIn.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "x"));
		iSize = int(cIn.Length());
		// Allocate the output vector
		cOut.Initialise(size_t(iSize));
		for(iIter = 0; iIter < iSize; ++iIter)
		{	// Calculate the error function
			cOut.GetElement(iIter).Initialise(
				StdExtensions::Erf(cIn.GetElement(iIter).GetAsDouble()));
		}
		pOut = cOut.ExportAsSEXP();
	}
	catch(StdExtensions::Exception cXa)
	{
		// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(...)
	{
		// Write error if a non-standard exception is thrown
		Rf_error("unknown error");
	}
	return pOut;
}

// Function to perform simple multidimensional Monte Carlo integration
//
//	func				-	Integrand function
//	lower				-	Lower limit of integration
//	upper				-	Upper limit of integration
//	initial.step		-	Number of runs to do in the first step
//	step.size			-	Number of runs to do in one time step
//	max.rel.var			-	Maximum relative variance before stopping MC integration
//	max.runs			-	Maximum number of runs before stopping MC integration
//	...					-	Extra parameters to pass to the integrand function
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP MCIntegration(SEXP pArgs)
#else
	extern "C" SEXP MCIntegration(SEXP pArgs)
#endif
{
	SEXP pOut = R_NilValue;
	RInterface::RRandom cRand;
	RInterface::RDoubleVector cOut, *pMCInputs, *pLower, *pUpper, cMCOutputs;
	RInterface::RIntegerVector *pMargin;
	StdExtensions::ExtDouble cMaxRuns;
	RInterface::RList cArgs, cApplyArgs;
	RInterface::RList::RListIterator cIter;
	RInterface::RFunction *pFunc;
	char pCharBuff[LD_STRBUFF_SIZE + 1];
	int iInitStep, iStepSize, iNumVariates, iTotalNum, iIter, pDims[2];
	double dNewVal, dMaxRelVar, dCurMean, dCurVar, dVolume, dVal, dVar, dDelta, dMTwo;
	pMCInputs = pLower = pUpper = NULL;
	pMargin = NULL;
	pFunc = NULL;
	pCharBuff[0] = '\0';
	iNumVariates = iTotalNum = iIter = pDims[0] = pDims[1] = 0;
	dNewVal = dCurMean = dCurVar= dVolume = dVal = dVar = dDelta = dMTwo = 0.0;
	iInitStep = iStepSize = -1;
	dMaxRelVar = -1.0;
	try
	{	// Import the arguments passed to the function
		cArgs.FillFromExternalParameters(pArgs);
		// Initialise the apply parameter list
		//	set MC input parameters
		cApplyArgs.AddElementToBack(RInterface::RDoubleVector(), "X");
		cIter.Initialise(cApplyArgs.CreateNewIteratorBack());
		pMCInputs = dynamic_cast<RInterface::RDoubleVector *>(&cIter.GetObject());
		//	set margin value
		cApplyArgs.AddElementToBack(RInterface::RIntegerVector(), "MARGIN");
		cIter.Initialise(cApplyArgs.CreateNewIteratorBack());
		pMargin = dynamic_cast<RInterface::RIntegerVector *>(&cIter.GetObject());
		pMargin->Initialise(1);
		pMargin->GetElement(0).Initialise(2);
		//	set function
		cApplyArgs.AddElementToBack(RInterface::RFunction(), "FUN");
		cIter.Initialise(cApplyArgs.CreateNewIteratorBack());
		pFunc = dynamic_cast<RInterface::RFunction *>(&cIter.GetObject());
		// Iterate over the input arguments
		for(cIter.Initialise(cArgs.CreateNewIteratorFront()); cIter.IsValid(); cIter.Increment())
		{
			cIter.GetTitle(pCharBuff, LD_STRBUFF_SIZE);
			if(strncmp(pCharBuff, "func", LD_STRBUFF_SIZE) == 0)
			{	// Set the integrand
				pFunc->Initialise(
					dynamic_cast<RInterface::RFunction &>(cIter.GetObject()));
			}
			else if(strncmp(pCharBuff, "lower", LD_STRBUFF_SIZE) == 0)
			{	// Set the lower integration limit
				pLower = dynamic_cast<RInterface::RDoubleVector *>(&cIter.GetObject());
			}
			else if(strncmp(pCharBuff, "upper", LD_STRBUFF_SIZE) == 0)
			{	// Set the upper integration limit
				pUpper = dynamic_cast<RInterface::RDoubleVector *>(&cIter.GetObject());
			}
			else if(strncmp(pCharBuff, "initial.step", LD_STRBUFF_SIZE) == 0)
			{	// Set the initial step size
				iInitStep = (dynamic_cast<RInterface::RIntegerVector &>(cIter.GetObject())).GetElement(0).GetAsInteger();
			}
			else if(strncmp(pCharBuff, "step.size", LD_STRBUFF_SIZE) == 0)
			{	// Set the step size
				iStepSize = (dynamic_cast<RInterface::RIntegerVector &>(cIter.GetObject())).GetElement(0).GetAsInteger();
			}
			else if(strncmp(pCharBuff, "max.rel.var", LD_STRBUFF_SIZE) == 0)
			{	// Set the maximum relative variance
				dMaxRelVar = (dynamic_cast<RInterface::RDoubleVector &>(cIter.GetObject())).GetElement(0).GetAsDouble();
			}
			else if(strncmp(pCharBuff, "max.runs", LD_STRBUFF_SIZE) == 0)
			{
				cMaxRuns = (dynamic_cast<RInterface::RDoubleVector &>(cIter.GetObject())).GetElement(0);
			}
			else
			{	// Add extra parameters to the apply function parameter list
				cApplyArgs.AddElementToBack(cIter.GetObject(), pCharBuff);
			}
		}
		// Test to ensure that parameter inputs set correctly
		if(pLower == NULL || pUpper == NULL)
		{
			StdExtensions::RaiseException("upper or lower limits of integration not set", __LINE__, "MCIntegration(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else if(iInitStep < 2)
		{
			StdExtensions::RaiseException("invalid initial step size set", __LINE__, "MCIntegration(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else if(iStepSize < 2)
		{
			StdExtensions::RaiseException("invalid step size set", __LINE__, "MCIntegration(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else if(dMaxRelVar <= 0.0)
		{
			StdExtensions::RaiseException("invalid maximum variation of integration set", __LINE__, "MCIntegration(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else if(!cMaxRuns.IsMissingValue() && cMaxRuns.LessThan(2.0).GetAsBool())
		{
			StdExtensions::RaiseException("invalid value for the maximum number of iterations", __LINE__, "MCIntegration(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		// Calculate the number of variates to integrate over
		iNumVariates = StdExtensions::Max(int(pLower->Length()), int(pUpper->Length()));
		// Calculate the volume of integration
		dVolume = 0.0;
		for(iIter = 0; iIter < iNumVariates; ++iIter)
		{
			dVolume += fabs(pLower->GetElement(iIter).GetAsDouble() - pUpper->GetElement(iIter).GetAsDouble());
		}
		// Ensure that step does not overshoot maximum run size
		if(!cMaxRuns.IsPositiveInfinity())
		{
			if(iInitStep > cMaxRuns.GetAsInteger())
			{
				iInitStep = cMaxRuns.GetAsInteger();
			}
		}
		// ------ MAIN INTEGRATION ALGORITHM ------
		// Make an initial number of runs if requested
		if(iInitStep > 0)
		{
			// Initialise the Monte Carlo integration input vector
			pMCInputs->Initialise(size_t(iNumVariates * iInitStep));
			pDims[0] = iNumVariates;
			pDims[1] = iInitStep;
			pMCInputs->SetDimensionStructure(2, pDims, NULL);
			// Make an initial number of runs
			// Create a set of the integration inputs
			for(pDims[0] = 0; pDims[0] < iNumVariates; ++pDims[0])
			{
				for(pDims[1] = 0; pDims[1] < iInitStep; ++pDims[1])
				{
					pMCInputs->GetElement(pDims).Initialise(
						cRand.Uniform(
							pLower->GetElement(pDims[0]).GetAsDouble(),
							pUpper->GetElement(pDims[0]).GetAsDouble()));
				}
			}
			// Call the apply function with the integration inputs
			cMCOutputs.ImportFromSEXP(RInterface::CallRFunction("apply", cApplyArgs));
			// Calculate the new sample mean
			for(iIter = 0; iIter < iInitStep; ++iIter)
			{
				if(cMCOutputs.GetElement(iIter).IsSpecialValue())
				{
					StdExtensions::RaiseException("NA, NaN, +Inf, or -Inf returned by integrand", __LINE__, "MCIntegration(SEXP)", __FILE__, StdExtensions::e_RangeError);
				}
				else
				{
					dNewVal = cMCOutputs.GetElement(iIter).GetAsDouble();
					dDelta = dNewVal - dCurMean;
					dCurMean += dDelta / double(iIter + 1);
					dMTwo += dDelta * (dNewVal - dCurMean);
				}
			}
			// Calculate the new sample variance
			dCurVar = dMTwo / double(iInitStep - 1);
			// Calculate the current integration value
			dVal = dVolume * dCurMean;
			// Calculate the current estimate of the variance of integration
			dVar = (StdExtensions::Sqrd(dVolume) / double(iInitStep)) * dCurVar;
		}
		// Initialise the Monte Carlo integration input vector
		pMCInputs->Initialise(size_t(iNumVariates * iStepSize));
		pDims[0] = iNumVariates;
		pDims[1] = iStepSize;
		pMCInputs->SetDimensionStructure(2, pDims, NULL);
		// Keep making sets of runs until relative variance is smaller than acceptable limit
		// or until the maximum number of runs is reached
		for(iTotalNum = iInitStep;
			dVar / fabs(dVal) >= dMaxRelVar &&
			cMaxRuns.GreaterThan(double(iTotalNum)).GetAsBool();
			iTotalNum += iStepSize)
		{
			// Ensure that step does not overshoot maximum run size
			if(!cMaxRuns.IsPositiveInfinity())
			{
				if(iTotalNum + iStepSize > cMaxRuns.GetAsInteger())
				{
					iStepSize = cMaxRuns.GetAsInteger() - iTotalNum;
				}
			}
			// Create a set of the integration inputs
			for(pDims[0] = 0; pDims[0] < iNumVariates; ++pDims[0])
			{
				for(pDims[1] = 0; pDims[1] < iStepSize; ++pDims[1])
				{
					pMCInputs->GetElement(pDims).Initialise(
						cRand.Uniform(
							pLower->GetElement(pDims[0]).GetAsDouble(),
							pUpper->GetElement(pDims[0]).GetAsDouble()));
				}
			}
			// Call the apply function with the integration inputs
			cMCOutputs.ImportFromSEXP(RInterface::CallRFunction("apply", cApplyArgs));
			// Calculate the new sample mean
			for(iIter = 0; iIter < iStepSize; ++iIter)
			{
				if(cMCOutputs.GetElement(iIter).IsSpecialValue())
				{
					StdExtensions::RaiseException("NA, NaN, +Inf, or -Inf returned by integrand", __LINE__, "MCIntegration(SEXP)", __FILE__, StdExtensions::e_RangeError);
				}
				else
				{
					dNewVal = cMCOutputs.GetElement(iIter).GetAsDouble();
					dDelta = dNewVal - dCurMean;
					dCurMean += dDelta / double(iTotalNum + iIter + 1);
					dMTwo += dDelta * (dNewVal - dCurMean);
				}
			}
			// Calculate the current integration value
			dCurVar = dMTwo / double(iTotalNum + iStepSize - 1);
			dVal = dVolume * dCurMean;
			// Calculate the current estimate of the variance of integration
			dVar = (StdExtensions::Sqrd(dVolume) / double(iTotalNum + iStepSize - 1)) * dCurVar;
		}
		// ------ END OF MAIN INTEGRATION ALGORITHM ------
		// Initialise and set the elements of the output vector
		cOut.Initialise(3);
		cOut.GetElement(0).Initialise(dVal);
		cOut.GetElement(1).Initialise(dVar);
		cOut.GetElement(2).Initialise(double(iTotalNum));
		pOut = cOut.ExportAsSEXP();
	}
	catch(StdExtensions::Exception cXa)
	{	// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(std::bad_cast)
	{	// Write error if casting exception is thrown
		Rf_error("error thrown during pointer casting process");
	}
	catch(...)
	{	// Write error if non-standard exception is thrown
		Rf_error("unknown error");
	}
	return pOut;
}

// Function to calculate cell-to-cell transition probabilities based on different approximations
// to continuous Gaussian dispersal.  We use the two dimension Gaussian function used in Clark et
// al. 1999.  This function calculates transition probabilities for uneven grid.
//
//	x1					-	Vector of lower boundary coordinates on the x-dimension
//	x2					-	Vector of upper boundary coordinates on the x-dimension
//	y1					-	Vector of lower boundary coordinates on the y-dimension
//	y2					-	Vector of upper boundary coordinates on the y-dimension
//	alpha				-	Scalar value for the shape parameter of the Gaussian distribution
//	approx.method		-	Single string describing the approximation method to employ.  Must
//							be one of the following values:
//								"CC" - Centroid-to-centroid dispersal
//								"CA" - Centroid-to-area dispersal
//								"AC" - Area-to-centroid dispersal
//								"AA" - Area-to-area dispersal
//	boundary			-	Single string describing the boundary condition to apply.  Must be
//							one of the following values:
//								"restricting" - Restricting boundary conditions
//								"absorbing" - Absorbing boundary conditions
//								"periodic" - Periodic boundary conditions
//	max.prob			-	The maximum probability of any extra arena length
//							before stopping condition is met when calculating
//							torus boundary conditions
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP GaussianLatticeTransition(SEXP pArgs)
#else
	extern "C" SEXP GaussianLatticeTransition(SEXP pArgs)
#endif
{
	RInterface::RDoubleVector cX1, cX2, cY1, cY2, cAlpha, cOut, cTorusMaxProb;
	RInterface::RStringVector cMethod, cBoundary;
	LatticeDispersal::ApproxMethod eMethod;
	LatticeDispersal::BoundaryConditions eBoundary;
	double dAlpha, dRunTot, dOldRunTot, dCurrent, dArenaXMin, dArenaXMax, dArenaYMin, dArenaYMax;
	double dArenaWidth, dArenaHeight, dTorusMaxProb;
	int iNumCells, pDims[2], iIter, iSubIter;
	SEXP pOut = R_NilValue;
	dAlpha = dRunTot = dOldRunTot = dCurrent = dArenaXMin = dArenaXMax = dArenaYMin = dArenaYMax =
		dArenaWidth = dArenaHeight = dTorusMaxProb = 0.0;
	iNumCells = pDims[0] = pDims[1] = iIter = iSubIter = 0;
	eBoundary = LatticeDispersal::e_Absorbing;
	eMethod = LatticeDispersal::e_AA;
	try
	{	// Import boundary information from the input arguments
		cX1.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "x1"));
		cX2.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "x2"));
		cY1.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "y1"));
		cY2.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "y2"));
		// Calculate the number of cells
		iNumCells = StdExtensions::Max(
			StdExtensions::Max(int(cX1.Length()), int(cX2.Length())),
			StdExtensions::Max(int(cY1.Length()), int(cY2.Length())));
		// Calculate the minimum bounding rectangle of the cells
		dArenaXMin = StdExtensions::Min(
			cX1.GetElement(0).GetAsDouble(), cX2.GetElement(0).GetAsDouble());
		dArenaXMax = StdExtensions::Max(
			cX1.GetElement(0).GetAsDouble(), cX2.GetElement(0).GetAsDouble());
		dArenaYMin = StdExtensions::Min(
			cY1.GetElement(0).GetAsDouble(), cY2.GetElement(0).GetAsDouble());
		dArenaYMax = StdExtensions::Max(
			cY1.GetElement(0).GetAsDouble(), cY2.GetElement(0).GetAsDouble());
		for(iIter = 1; iIter < iNumCells; ++iIter)
		{	// Calculate x-coordinate minimums and maximums
			dArenaXMin = StdExtensions::Min(dArenaXMin,
				StdExtensions::Min(
					cX1.GetElement(iIter).GetAsDouble(), cX2.GetElement(iIter).GetAsDouble()));
			dArenaXMax = StdExtensions::Max(dArenaXMax,
				StdExtensions::Max(
					cX1.GetElement(iIter).GetAsDouble(), cX2.GetElement(iIter).GetAsDouble()));
			// Calculate y-coordinate minimums and maximums
			dArenaYMin = StdExtensions::Min(dArenaYMin,
				StdExtensions::Min(
					cY1.GetElement(iIter).GetAsDouble(), cY2.GetElement(iIter).GetAsDouble()));
			dArenaYMax = StdExtensions::Max(dArenaYMax,
				StdExtensions::Max(
					cY1.GetElement(iIter).GetAsDouble(), cY2.GetElement(iIter).GetAsDouble()));
		}
		// Calculate the height and width of the bounding arena
		dArenaWidth = dArenaXMax - dArenaXMin;
		dArenaHeight = dArenaYMax - dArenaYMin;
		// Create output matrix
		cOut.Initialise(size_t(StdExtensions::Sqrd(iNumCells + 1)));
		pDims[0] = pDims[1] = iNumCells + 1;
		cOut.SetDimensionStructure(2, pDims);
		// Import alpha information from the input arguments
		cAlpha.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "alpha"));
		dAlpha = cAlpha.GetElement(0).GetAsDouble();
		if(dAlpha <= 0.0)
		{	// Ensure alpha is contained within acceptable boundaries
			StdExtensions::RaiseException("invalid value for alpha", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		// Import approximation method from the input arguments
		cMethod.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "approx.method"));
		// Import boundary information from the input arguments
		cBoundary.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "boundary"));
		// Set approximation method
		if(cMethod.GetElement(0).Equal("CC").GetAsBool())
		{	// Centroid-to-centroid dispersal
			eMethod = LatticeDispersal::e_CC;
		}
		else if(cMethod.GetElement(0).Equal("CA").GetAsBool())
		{	// Centroid-to-area dispersal
			eMethod = LatticeDispersal::e_CA;
		}
		else if(cMethod.GetElement(0).Equal("AC").GetAsBool())
		{	// Area-to-centroid dispersal
			eMethod = LatticeDispersal::e_AC;
		}
		else if(cMethod.GetElement(0).Equal("AA").GetAsBool())
		{	// Area-to-area dispersal
			eMethod = LatticeDispersal::e_AA;
		}
		else
		{	// Raise exception if unknown approximation method selected
			StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		// Set boundary condition
		if(cBoundary.GetElement(0).Equal("restricting").GetAsBool())
		{	// Resticting boundary conditions
			eBoundary = LatticeDispersal::e_Restricting;
		}
		else if(cBoundary.GetElement(0).Equal("absorbing").GetAsBool())
		{	// Absorbing boundary conditions
			eBoundary = LatticeDispersal::e_Absorbing;
		}
		else if(cBoundary.GetElement(0).Equal("periodic").GetAsBool())
		{	// Periodic boundary conditions
			eBoundary = LatticeDispersal::e_Periodic;
			// Set the torus parameters
			cTorusMaxProb.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "max.prob"));
			if((dTorusMaxProb = cTorusMaxProb.GetElement(0).GetAsDouble()) <= 0.0)
			{	// Invalid maximum probability parameters
				StdExtensions::RaiseException("maximum stopping probability for periodic calculation must be greater than zero", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
			// Ensure that all the area has been used - that there are no gaps in the
			// arena rectangle
			dRunTot = dArenaWidth * dArenaHeight;
			for(iIter = 0; iIter < iNumCells; ++iIter)
			{
				dRunTot -= fabs(cX1.GetElement(iIter).GetAsDouble() - 
					cX2.GetElement(iIter).GetAsDouble()) *
					fabs(cY1.GetElement(iIter).GetAsDouble() -
					cY2.GetElement(iIter).GetAsDouble());
			}
			// Check to see whether the running total is within a very
			// narrow band around 0
			if(dRunTot < -DBL_EPSILON || dRunTot > DBL_EPSILON)
			{
				StdExtensions::RaiseException("missing areas defined within periodic boundary", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
		}
		else
		{	// Raise exception if unknown boundary conditions selected
			StdExtensions::RaiseException("unknown boundary condition selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		if(eBoundary == LatticeDispersal::e_Absorbing &&
			(eMethod == LatticeDispersal::e_CC || eMethod == LatticeDispersal::e_AC))
		{	// Check for incompatible boundary conditions and approximation methods
			StdExtensions::RaiseException("unable to calculate centroid-to-centroid or area-to-centroid transition probabilitites with absorbing boundary conditions", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else
		{	// Iterate over the output - calculate the transition probabilities
			for(pDims[0] = 0; pDims[0] < iNumCells; ++pDims[0])
			{
				dRunTot = 0.0;		// Set the running total to 0.0
				for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
				{
					switch(eMethod)
					{
					case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
						dCurrent = CCGaussTrans(
							cX1.GetElement(pDims[0]).GetAsDouble(),
							cX2.GetElement(pDims[0]).GetAsDouble(),
							cY1.GetElement(pDims[0]).GetAsDouble(),
							cY2.GetElement(pDims[0]).GetAsDouble(),
							cX1.GetElement(pDims[1]).GetAsDouble(),
							cX2.GetElement(pDims[1]).GetAsDouble(),
							cY1.GetElement(pDims[1]).GetAsDouble(),
							cY2.GetElement(pDims[1]).GetAsDouble(),
							dAlpha);
						break;
					case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
						dCurrent = CAGaussTrans(
							cX1.GetElement(pDims[0]).GetAsDouble(),
							cX2.GetElement(pDims[0]).GetAsDouble(),
							cY1.GetElement(pDims[0]).GetAsDouble(),
							cY2.GetElement(pDims[0]).GetAsDouble(),
							cX1.GetElement(pDims[1]).GetAsDouble(),
							cX2.GetElement(pDims[1]).GetAsDouble(),
							cY1.GetElement(pDims[1]).GetAsDouble(),
							cY2.GetElement(pDims[1]).GetAsDouble(),
							dAlpha);
						break;
					case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
						dCurrent = ACGaussTrans(
							cX1.GetElement(pDims[0]).GetAsDouble(),
							cX2.GetElement(pDims[0]).GetAsDouble(),
							cY1.GetElement(pDims[0]).GetAsDouble(),
							cY2.GetElement(pDims[0]).GetAsDouble(),
							cX1.GetElement(pDims[1]).GetAsDouble(),
							cX2.GetElement(pDims[1]).GetAsDouble(),
							cY1.GetElement(pDims[1]).GetAsDouble(),
							cY2.GetElement(pDims[1]).GetAsDouble(),
							dAlpha);
						break;
					case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
						dCurrent = AAGaussTrans(
							cX1.GetElement(pDims[0]).GetAsDouble(),
							cX2.GetElement(pDims[0]).GetAsDouble(),
							cY1.GetElement(pDims[0]).GetAsDouble(),
							cY2.GetElement(pDims[0]).GetAsDouble(),
							cX1.GetElement(pDims[1]).GetAsDouble(),
							cX2.GetElement(pDims[1]).GetAsDouble(),
							cY1.GetElement(pDims[1]).GetAsDouble(),
							cY2.GetElement(pDims[1]).GetAsDouble(),
							dAlpha);
						break;
					default:		// Raise exception if unknown approximation method selected
						StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
					}
					cOut.GetElement(pDims).Initialise(dCurrent);
					dRunTot += dCurrent;
				}
				// Apply edge correction where neccessary
				switch(eBoundary)
				{
				case LatticeDispersal::e_Restricting:
					// Normalise the cell values
					for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
					{
						cOut.GetElement(pDims).Initialise(
							cOut.GetElement(pDims).Divide(dRunTot));
					}
					// Set the absorbing state to zero
					cOut.GetElement(pDims).Initialise(0.0);
					break;
				case LatticeDispersal::e_Absorbing:
					switch(eMethod)
					{
					case LatticeDispersal::e_CC:
					case LatticeDispersal::e_AC:
						// Throw error if absorbing boundary conditions used with centroid-to-centorid or area-to-centroid dispersal
						StdExtensions::RaiseException("unable to calculate centroid-to-centroid or area-to-centroid transition probabilitites with absorbing boundary conditions", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
						break;
					case LatticeDispersal::e_CA:
					case LatticeDispersal::e_AA:
						// Set the absorbing state to the remainder of the probability distribution
						cOut.GetElement(pDims).Initialise(1.0 - dRunTot);
						break;
					default:
						StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
					}
					break;
				case LatticeDispersal::e_Periodic:
					// ------ PERIODIC BONDARY CORRECTION ------
					// Increase the radius of simulation arenas until running total
					// close to one
					dOldRunTot = dRunTot + dTorusMaxProb;
					for(iIter = 1; CheckRunTot(dRunTot, dOldRunTot, dTorusMaxProb, eMethod); ++iIter)
					{	// Reset the running total
						dOldRunTot = dRunTot;
						dRunTot = 0.0;
						// Iterate over the possible destination cells
						for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
						{	// Set the current value
							dCurrent = cOut.GetElement(pDims).GetAsDouble();
							// Calculate the probability weights at the corners
							switch(eMethod)
							{
							case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
								dCurrent += CCGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										dAlpha) +
									CCGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										dAlpha) +
									CCGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha) +
									CCGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha);
								break;
							case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
								dCurrent += CAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										dAlpha) +
									CAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										dAlpha) +
									CAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha) +
									CAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha);
								break;
							case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
								dCurrent += ACGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										dAlpha) +
									ACGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										dAlpha) +
									ACGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha) +
									ACGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha);
								break;
							case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
								dCurrent += AAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										dAlpha) +
									AAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble(),
										cX2.GetElement(pDims[1]).GetAsDouble(),
										cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										dAlpha) +
									AAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha) +
									AAGaussTrans(
										cX1.GetElement(pDims[0]).GetAsDouble(),
										cX2.GetElement(pDims[0]).GetAsDouble(),
										cY1.GetElement(pDims[0]).GetAsDouble(),
										cY2.GetElement(pDims[0]).GetAsDouble(),
										cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										cY1.GetElement(pDims[1]).GetAsDouble(),
										cY2.GetElement(pDims[1]).GetAsDouble(),
										dAlpha);
								break;
							default:		// Raise exception if unknown approximation method selected
								StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
							}
							// Iterate over one edge of the simulation arena grid
							for(iSubIter = 1; iSubIter < iIter; ++iSubIter)
							{
								switch(eMethod)
								{
								case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
									dCurrent += CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha) +
										CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha);
									break;
								case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
									dCurrent += CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha) +
										CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha);
									break;
								case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
									dCurrent += ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha) +
										ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha);
									break;
								case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
									dCurrent += AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											dAlpha) +
										AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha) +
										AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											dAlpha);
									break;
								default:		// Raise exception if unknown approximation method selected
									StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
								}
							}
							// Iterate over the other edge of the simulation arena grid
							for(iSubIter = 1; iSubIter <= iIter; ++iSubIter)
							{
								switch(eMethod)
								{
								case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
									dCurrent += CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha) +
										CCGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha);
									break;
								case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
									dCurrent += CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha) +
										CAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha);
									break;
								case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
									dCurrent += ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha) +
										ACGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha);
									break;
								case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
									dCurrent += AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											dAlpha) +
										AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha) +
										AAGaussTrans(
											cX1.GetElement(pDims[0]).GetAsDouble(),
											cX2.GetElement(pDims[0]).GetAsDouble(),
											cY1.GetElement(pDims[0]).GetAsDouble(),
											cY2.GetElement(pDims[0]).GetAsDouble(),
											cX1.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cX2.GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											cY1.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cY2.GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											dAlpha);
									break;
								default:		// Raise exception if unknown approximation method selected
									StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
								}
							}
							// Set the current element
							cOut.GetElement(pDims).Initialise(dCurrent);
							dRunTot += dCurrent;
						}
					}
					// Final normalisation
					for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
					{
						cOut.GetElement(pDims).Initialise(
							cOut.GetElement(pDims).Divide(dRunTot));
					}
					// Set the absorbing state to zero
					cOut.GetElement(pDims).Initialise(0.0);
					// ------ END OF PERIODIC BOUNDARY CORRECTION ------
					break;
				default:
					// Raise exception if unknown boundary conditions selected
					StdExtensions::RaiseException("unknown boundary condition selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
				}
			}
			// Set the transition probabilities for the absorbing state
			for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
			{
				cOut.GetElement(pDims).Initialise(0.0);
			}
			cOut.GetElement(pDims).Initialise(1.0);
			// Export the transition matrix
			pOut = cOut.ExportAsSEXP();
		}
	}
	catch(StdExtensions::Exception cXa)
	{
		// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(...)
	{
		// Write error if a non-standard exception is thrown
		Rf_error("unknown error");
	}
	return pOut;
}

// Function to calculate cell-to-cell transition probabilities based on different approximations
// to continuous dispersal.  This function calculates transition probabilities for uneven grid.
//
//	x1					-	Vector of lower boundary coordinates on the x-dimension
//	x2					-	Vector of upper boundary coordinates on the x-dimension
//	y1					-	Vector of lower boundary coordinates on the y-dimension
//	y2					-	Vector of upper boundary coordinates on the y-dimension
//	func				-	Dispersal function for the description of the two dimensional dispersal kernel
//	approx.method		-	Single string describing the approximation method to employ.  Must
//							be one of the following values:
//								"CC" - Centroid-to-centroid dispersal
//								"CA" - Centroid-to-area dispersal
//								"AC" - Area-to-centroid dispersal
//								"AA" - Area-to-area dispersal
//	boundary			-	Single string describing the boundary condition to apply.  Must be
//							one of the following values:
//								"restricting" - Restricting boundary conditions
//								"absorbing" - Absorbing boundary conditions
//								"periodic" - Periodic boundary conditions
//	max.prob			-	The maximum probability of any extra arena length
//							before stopping condition is met when calculating
//							torus boundary conditions
//	initial.step		-	MC integration parameter: number of runs to do in the first step
//	step.size			-	MC integration parameter: number of runs to complete in each time step
//	max.rel.var			-	MC integration parameter: maximum relative variance before stopping MC integration
//	max.runs			-	MC integration parameter: maximum number of runs before stopping MC integration
//	params				-	Parameter vector used in dispersal calculation
//	disp.funcs			-	List of dispersal function wrappers
//	...					-	Extra parameters to pass to the dispersal function
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP LatticeTransition(SEXP pArgs)
#else
	extern "C" SEXP LatticeTransition(SEXP pArgs)
#endif
{
	RInterface::RDoubleVector *pX1, *pX2, *pY1, *pY2, cOut, cTorusMaxProb;
	RInterface::RFunction cWrap;
	RInterface::RStringVector *pMethod, *pBoundary;
	RInterface::RList cArgs, cExtraParams, *pFuncList;
	RInterface::RList::RListIterator cIter;
	//RInterface::RFunction *pFunc;
	LatticeDispersal::ApproxMethod eMethod;
	LatticeDispersal::BoundaryConditions eBoundary;
	double dRunTot, dOldRunTot, dCurrent, dArenaXMin, dArenaXMax, dArenaYMin, dArenaYMax;
	double dArenaWidth, dArenaHeight, dTorusMaxProb;
	char pCharBuff[LD_STRBUFF_SIZE + 1];
	int iNumCells, pDims[2], iIter, iSubIter;
	SEXP pOut = R_NilValue;
	pX1 = pX2 = pY1 = pY2 = NULL;
	pMethod = pBoundary = NULL;
	pFuncList = NULL;
	//pFunc = NULL;
	dRunTot = dOldRunTot = dCurrent = dArenaXMin = dArenaXMax = dArenaYMin = dArenaYMax =
		dArenaWidth = dArenaHeight = dTorusMaxProb = 0.0;
	iNumCells = pDims[0] = pDims[1] = iIter = iSubIter = 0;
	dTorusMaxProb = -1.0;
	eMethod = LatticeDispersal::e_AA;
	eBoundary = LatticeDispersal::e_Absorbing;
	try
	{	// Import the arguments passed to the function
		cArgs.FillFromExternalParameters(pArgs);
		// Iterate over the input arguments
		for(cIter.Initialise(cArgs.CreateNewIteratorFront()); cIter.IsValid(); cIter.Increment())
		{
			cIter.GetTitle(pCharBuff, LD_STRBUFF_SIZE);
			if(strncmp(pCharBuff, "x1", LD_STRBUFF_SIZE) == 0)
			{	// Set the lower boundary x-coordinates
				pX1 = dynamic_cast<RInterface::RDoubleVector *>(&cIter.GetObject());
			}
			else if(strncmp(pCharBuff, "x2", LD_STRBUFF_SIZE) == 0)
			{	// Set the upper boundary x-coordinate
				pX2 = dynamic_cast<RInterface::RDoubleVector *>(&cIter.GetObject());
			}
			else if(strncmp(pCharBuff, "y1", LD_STRBUFF_SIZE) == 0)
			{	// Set the lower boundary y-coordinate
				pY1 = dynamic_cast<RInterface::RDoubleVector *>(&cIter.GetObject());
			}
			else if(strncmp(pCharBuff, "y2", LD_STRBUFF_SIZE) == 0)
			{	// Set the upper boundary y-coordinate
				pY2 = dynamic_cast<RInterface::RDoubleVector *>(&cIter.GetObject());
			}
			else if(strncmp(pCharBuff, "func", LD_STRBUFF_SIZE) == 0)
			{	// Set the dispersal function
				//pFunc = dynamic_cast<RInterface::RFunction *>(&cIter.GetObject());
				cExtraParams.AddElementToBack(cIter.GetObject(), "func.in");
			}
			else if(strncmp(pCharBuff, "approx.method", LD_STRBUFF_SIZE) == 0)
			{	// Set the approximation method
				pMethod = dynamic_cast<RInterface::RStringVector *>(&cIter.GetObject());
				pMethod->GetElement(0).RetrieveCString(pCharBuff, LD_STRBUFF_SIZE);
				if(strncmp(pCharBuff, "CC", LD_STRBUFF_SIZE) == 0)
				{	// Centroid-to-centroid dispersal
					eMethod = LatticeDispersal::e_CC;
				}
				else if(strncmp(pCharBuff, "CA", LD_STRBUFF_SIZE) == 0)
				{	// Centroid-to-area dispersal
					eMethod = LatticeDispersal::e_CA;
				}
				else if(strncmp(pCharBuff, "AC", LD_STRBUFF_SIZE) == 0)
				{	// Area-to-centroid dispersal
					eMethod = LatticeDispersal::e_AC;
				}
				else if(strncmp(pCharBuff, "AA", LD_STRBUFF_SIZE) == 0)
				{	// Area-to-area dispersal
					eMethod = LatticeDispersal::e_AA;
				}
				else
				{	// Unknown approximation method
					StdExtensions::RaiseException("unknown approximation method", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
				}
			}
			else if(strncmp(pCharBuff, "boundary", LD_STRBUFF_SIZE) == 0)
			{	// Set the boundary condition
				pBoundary = dynamic_cast<RInterface::RStringVector *>(&cIter.GetObject());
				pBoundary->GetElement(0).RetrieveCString(pCharBuff, LD_STRBUFF_SIZE);
				if(strncmp(pCharBuff, "restricting", LD_STRBUFF_SIZE) == 0)
				{	// Restricting boundary conditions
					eBoundary = LatticeDispersal::e_Restricting;
				}
				else if(strncmp(pCharBuff, "absorbing", LD_STRBUFF_SIZE) == 0)
				{	// Absorbing boundary conditions
					eBoundary = LatticeDispersal::e_Absorbing;
				}
				else if(strncmp(pCharBuff, "periodic", LD_STRBUFF_SIZE) == 0)
				{	// Periodic boundary conditions
					eBoundary = LatticeDispersal::e_Periodic;
				}
				else
				{	// Unknown boundary condition
					StdExtensions::RaiseException("unknown boundary condition", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
				}
			}
			else if(strncmp(pCharBuff, "max.prob", LD_STRBUFF_SIZE) == 0)
			{	// Set the maximum probability stopping condition for infinite series
				dTorusMaxProb = (dynamic_cast<RInterface::RDoubleVector &>(cIter.GetObject())).GetElement(0).GetAsDouble();
			}
			else if(strncmp(pCharBuff, "initial.step", LD_STRBUFF_SIZE) == 0)
			{	// Set the initial step size for Monte Carlo integration
				cExtraParams.AddElementToBack(cIter.GetObject(), "initial.step");
			}
			else if(strncmp(pCharBuff, "step.size", LD_STRBUFF_SIZE) == 0)
			{	// Set the step size in the Monte Carlo integration
				cExtraParams.AddElementToBack(cIter.GetObject(), "step.size");
			}
			else if(strncmp(pCharBuff, "max.rel.var", LD_STRBUFF_SIZE) == 0)
			{	// Set the maximum relative variance in the Monte Carlo integration
				cExtraParams.AddElementToBack(cIter.GetObject(), "max.rel.var");
			}
			else if(strncmp(pCharBuff, "max.runs", LD_STRBUFF_SIZE) == 0)
			{	// Set the maximum number of runs in the Monte Carlo integration
				cExtraParams.AddElementToBack(cIter.GetObject(), "max.runs");
			}
			else if(strncmp(pCharBuff, "params", LD_STRBUFF_SIZE) == 0)
			{	// Set the parameters vector
				cExtraParams.AddElementToBack(cIter.GetObject(), "params");
			}
			else if(strncmp(pCharBuff, "disp.funcs", LD_STRBUFF_SIZE) == 0)
			{	// Set the dispersal function wrapper list
				pFuncList = dynamic_cast<RInterface::RList *>(&cIter.GetObject());
			}
			else
			{	// Extra parameter
				cExtraParams.AddElementToBack(cIter.GetObject(), pCharBuff);
			}
		}
		// Make sure that the list of function wrappers are set
		if(pFuncList == NULL)
		{
			StdExtensions::RaiseException("list of function wrappers is not set", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else
		{	// Fund the relevant dispersal function wrapper
			switch(eMethod)
			{
				// Centroid-to-centroid dispersal
			case LatticeDispersal::e_CC:
				cWrap.Initialise(
					dynamic_cast<RInterface::RFunction &>(pFuncList->GetElement("CC")));
				break;
				// Centroid-to-area dispersal
			case LatticeDispersal::e_CA:
				cWrap.Initialise(
					dynamic_cast<RInterface::RFunction &>(pFuncList->GetElement("CA")));
				break;
				// Area-to-centroid dispersal
			case LatticeDispersal::e_AC:
				cWrap.Initialise(
					dynamic_cast<RInterface::RFunction &>(pFuncList->GetElement("AC")));
				break;
				// Area-to-area dispersal
			case LatticeDispersal::e_AA:
				cWrap.Initialise(
					dynamic_cast<RInterface::RFunction &>(pFuncList->GetElement("AA")));
				break;
			default:
				StdExtensions::RaiseException("unknown approximation method", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
		}
		// Make sure that cell boundaries are set
		if(pX1 == NULL || pX2 == NULL || pY1 == NULL || pY2 == NULL)
		{
			StdExtensions::RaiseException("one or more cell boundary coordinate parameters not set", __LINE__, "StdExtensions::LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		// Calculate the number of cells
		iNumCells = StdExtensions::Max(
			StdExtensions::Max(int(pX1->Length()), int(pX2->Length())),
			StdExtensions::Max(int(pY1->Length()), int(pY2->Length())));
		// Calculate the minimum bounding rectangle of the cells
		dArenaXMin = StdExtensions::Min(
			pX1->GetElement(0).GetAsDouble(), pX2->GetElement(0).GetAsDouble());
		dArenaXMax = StdExtensions::Max(
			pX1->GetElement(0).GetAsDouble(), pX2->GetElement(0).GetAsDouble());
		dArenaYMin = StdExtensions::Min(
			pY1->GetElement(0).GetAsDouble(), pY2->GetElement(0).GetAsDouble());
		dArenaYMax = StdExtensions::Max(
			pY1->GetElement(0).GetAsDouble(), pY2->GetElement(0).GetAsDouble());
		for(iIter = 1; iIter < iNumCells; ++iIter)
		{	// Calculate x-coordinate minimums and maximums
			dArenaXMin = StdExtensions::Min(dArenaXMin,
				StdExtensions::Min(
					pX1->GetElement(iIter).GetAsDouble(), pX2->GetElement(iIter).GetAsDouble()));
			dArenaXMax = StdExtensions::Max(dArenaXMax,
				StdExtensions::Max(
					pX1->GetElement(iIter).GetAsDouble(), pX2->GetElement(iIter).GetAsDouble()));
			// Calculate y-coordinate minimums and maximums
			dArenaYMin = StdExtensions::Min(dArenaYMin,
				StdExtensions::Min(
					pY1->GetElement(iIter).GetAsDouble(), pY2->GetElement(iIter).GetAsDouble()));
			dArenaYMax = StdExtensions::Max(dArenaYMax,
				StdExtensions::Max(
					pY1->GetElement(iIter).GetAsDouble(), pY2->GetElement(iIter).GetAsDouble()));
		}
		// Calculate the height and width of the bounding arena
		dArenaWidth = dArenaXMax - dArenaXMin;
		dArenaHeight = dArenaYMax - dArenaYMin;
		// Create output matrix
		cOut.Initialise(size_t(StdExtensions::Sqrd(iNumCells + 1)));
		pDims[0] = pDims[1] = iNumCells + 1;
		cOut.SetDimensionStructure(2, pDims);
		// Periodic boundary conditions
		if(eBoundary == LatticeDispersal::e_Periodic)
		{
			if(dTorusMaxProb <= 0.0)
			{	// Invalid maximum probability parameters
				StdExtensions::RaiseException("invalid value for the maximum probability stopping condition parameter", __LINE__, "StdExtensions::LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
			// Ensure that all the area has been used - that there are no gaps in the
			// arena rectangle
			dRunTot = dArenaWidth * dArenaHeight;
			for(iIter = 0; iIter < iNumCells; ++iIter)
			{
				dRunTot -= fabs(pX1->GetElement(iIter).GetAsDouble() -
					pX2->GetElement(iIter).GetAsDouble()) *
					fabs(pY1->GetElement(iIter).GetAsDouble() -
					pY2->GetElement(iIter).GetAsDouble());
			}
			// Check to see whether the running total is within a very
			// narrow band around 0
			if(dRunTot < -DBL_EPSILON || dRunTot > DBL_EPSILON)
			{
				StdExtensions::RaiseException("missing areas defined within periodic boundary", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
		}
		if(eBoundary == LatticeDispersal::e_Absorbing &&
			(eMethod == LatticeDispersal::e_CC || eMethod == LatticeDispersal::e_AC))
		{	// Check for incomplete boundary conditions and approximation methods
			StdExtensions::RaiseException("unable to calculate centroid-to-centroid or area-to-centroid transition probabilities with absorbing boundary conditions", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else
		{	// Iterate over the output - calculate the transition probabilities
			for(pDims[0] = 0; pDims[0] < iNumCells; ++pDims[0])
			{
				dRunTot = 0.0;		// Set the running total to 0.0
				for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
				{
					switch(eMethod)
					{
					case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
						dCurrent = CCTrans(
							pX1->GetElement(pDims[0]).GetAsDouble(),
							pX2->GetElement(pDims[0]).GetAsDouble(),
							pY1->GetElement(pDims[0]).GetAsDouble(),
							pY2->GetElement(pDims[0]).GetAsDouble(),
							pX1->GetElement(pDims[1]).GetAsDouble(),
							pX2->GetElement(pDims[1]).GetAsDouble(),
							pY1->GetElement(pDims[1]).GetAsDouble(),
							pY2->GetElement(pDims[1]).GetAsDouble(),
							cWrap, cExtraParams);
						break;
					case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
						dCurrent = CATrans(
							pX1->GetElement(pDims[0]).GetAsDouble(),
							pX2->GetElement(pDims[0]).GetAsDouble(),
							pY1->GetElement(pDims[0]).GetAsDouble(),
							pY2->GetElement(pDims[0]).GetAsDouble(),
							pX1->GetElement(pDims[1]).GetAsDouble(),
							pX2->GetElement(pDims[1]).GetAsDouble(),
							pY1->GetElement(pDims[1]).GetAsDouble(),
							pY2->GetElement(pDims[1]).GetAsDouble(),
							cWrap, cExtraParams);
						break;
					case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
						dCurrent = ACTrans(
							pX1->GetElement(pDims[0]).GetAsDouble(),
							pX2->GetElement(pDims[0]).GetAsDouble(),
							pY1->GetElement(pDims[0]).GetAsDouble(),
							pY2->GetElement(pDims[0]).GetAsDouble(),
							pX1->GetElement(pDims[1]).GetAsDouble(),
							pX2->GetElement(pDims[1]).GetAsDouble(),
							pY1->GetElement(pDims[1]).GetAsDouble(),
							pY2->GetElement(pDims[1]).GetAsDouble(),
							cWrap, cExtraParams);
						break;
					case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
						dCurrent = AATrans(
							pX1->GetElement(pDims[0]).GetAsDouble(),
							pX2->GetElement(pDims[0]).GetAsDouble(),
							pY1->GetElement(pDims[0]).GetAsDouble(),
							pY2->GetElement(pDims[0]).GetAsDouble(),
							pX1->GetElement(pDims[1]).GetAsDouble(),
							pX2->GetElement(pDims[1]).GetAsDouble(),
							pY1->GetElement(pDims[1]).GetAsDouble(),
							pY2->GetElement(pDims[1]).GetAsDouble(),
							cWrap, cExtraParams);
						break;
					default:		// Raise exception if unknown approximation method selected
						StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
					}
					cOut.GetElement(pDims).Initialise(dCurrent);
					dRunTot += dCurrent;
				}
				// Apply edge correction where neccessary
				switch(eBoundary)
				{
				case LatticeDispersal::e_Restricting:
					// Normalise the cell values
					for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
					{
						cOut.GetElement(pDims).Initialise(
							cOut.GetElement(pDims).Divide(dRunTot));
					}
					// Set the absorbing state to zero
					cOut.GetElement(pDims).Initialise(0.0);
					break;
				case LatticeDispersal::e_Absorbing:
					switch(eMethod)
					{
					case LatticeDispersal::e_CC:
					case LatticeDispersal::e_AC:
						// Throw error if absorbing boundary conditions used with centroid-to-centroid or area-to-centroid dispersal
						StdExtensions::RaiseException("unable to calculate centroid-to-centroid or area-to-centroid transition probabilitites with absorbing boundary conditions", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
						break;
					case LatticeDispersal::e_CA:
					case LatticeDispersal::e_AA:
						// Set the absorbing state to the remainder of the probability distribution
						cOut.GetElement(pDims).Initialise(1.0 - dRunTot);
						break;
					default:
						StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
					}
					break;
				case LatticeDispersal::e_Periodic:
					// ------ PERIODIC BOUNDARY CORRECTION ------
					// Increase the radius of simulation arenas until running total
					// close to one
					dOldRunTot = dRunTot + dTorusMaxProb;
					for(iIter = 1; CheckRunTot(dRunTot, dOldRunTot, dTorusMaxProb, eMethod); ++iIter)
					{	// Reset the running total
						dOldRunTot = dRunTot;
						dRunTot = 0.0;
						// Iterate over the possible destination cells
						for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
						{	// Set the current value
							dCurrent = cOut.GetElement(pDims).GetAsDouble();
							// Calculate the probability weights at the corners
							switch(eMethod)
							{
							case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
								dCurrent += CCTrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble(),
									pX2->GetElement(pDims[1]).GetAsDouble(),
									pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
									pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
									cWrap, cExtraParams) +
								CCTrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble(),
									pX2->GetElement(pDims[1]).GetAsDouble(),
									pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
									pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
									cWrap, cExtraParams) +
								CCTrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
									pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
									pY1->GetElement(pDims[1]).GetAsDouble(),
									pY2->GetElement(pDims[1]).GetAsDouble(),
									cWrap, cExtraParams) +
								CCTrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
									pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
									pY1->GetElement(pDims[1]).GetAsDouble(),
									pY2->GetElement(pDims[1]).GetAsDouble(),
									cWrap, cExtraParams);
								break;
							case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
								dCurrent += CATrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble(),
									pX2->GetElement(pDims[1]).GetAsDouble(),
									pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
									pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
									cWrap, cExtraParams) +
								CATrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble(),
									pX2->GetElement(pDims[1]).GetAsDouble(),
									pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
									pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
									cWrap, cExtraParams) +
								CATrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
									pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
									pY1->GetElement(pDims[1]).GetAsDouble(),
									pY2->GetElement(pDims[1]).GetAsDouble(),
									cWrap, cExtraParams) +
								CATrans(
									pX1->GetElement(pDims[0]).GetAsDouble(),
									pX2->GetElement(pDims[0]).GetAsDouble(),
									pY1->GetElement(pDims[0]).GetAsDouble(),
									pY2->GetElement(pDims[0]).GetAsDouble(),
									pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
									pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
									pY1->GetElement(pDims[1]).GetAsDouble(),
									pY2->GetElement(pDims[1]).GetAsDouble(),
									cWrap, cExtraParams);
								break;
							case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
								dCurrent += ACTrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble(),
										pX2->GetElement(pDims[1]).GetAsDouble(),
										pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										cWrap, cExtraParams) +
									ACTrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble(),
										pX2->GetElement(pDims[1]).GetAsDouble(),
										pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										cWrap, cExtraParams) +
									ACTrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										pY1->GetElement(pDims[1]).GetAsDouble(),
										pY2->GetElement(pDims[1]).GetAsDouble(),
										cWrap, cExtraParams) +
									ACTrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										pY1->GetElement(pDims[1]).GetAsDouble(),
										pY2->GetElement(pDims[1]).GetAsDouble(),
										cWrap, cExtraParams);
								break;
							case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
								dCurrent += AATrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble(),
										pX2->GetElement(pDims[1]).GetAsDouble(),
										pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
										cWrap, cExtraParams) +
									AATrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble(),
										pX2->GetElement(pDims[1]).GetAsDouble(),
										pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
										cWrap, cExtraParams) +
									AATrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
										pY1->GetElement(pDims[1]).GetAsDouble(),
										pY2->GetElement(pDims[1]).GetAsDouble(),
										cWrap, cExtraParams) +
									AATrans(
										pX1->GetElement(pDims[0]).GetAsDouble(),
										pX2->GetElement(pDims[0]).GetAsDouble(),
										pY1->GetElement(pDims[0]).GetAsDouble(),
										pY2->GetElement(pDims[0]).GetAsDouble(),
										pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
										pY1->GetElement(pDims[1]).GetAsDouble(),
										pY2->GetElement(pDims[1]).GetAsDouble(),
										cWrap, cExtraParams);
								break;
							default:		// Raise exception if unknown approximation method selected
								StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
							}
							// Iterate over one edge of the simulation arena grid
							for(iSubIter = 1; iSubIter < iIter; ++iSubIter)
							{
								switch(eMethod)
								{
								case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
									dCurrent += CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
									dCurrent += CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
									dCurrent += ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
									dCurrent += AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams) +
										AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								default:		// Raise exception if unknown approximation method selected
									StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
								}
							}
							// Iterate over the other edge of the simulation arena grid
							for(iSubIter = 1; iSubIter <= iIter; ++iSubIter)
							{
								switch(eMethod)
								{
								case LatticeDispersal::e_CC:		// Calculate the centroid-to-centroid transition probabilities
									dCurrent += CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CCTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								case LatticeDispersal::e_CA:		// Calculate the centroid-to-area transition probabilities
									dCurrent += CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										CATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								case LatticeDispersal::e_AC:		// Calculate the area-to-centroid transition probabilities
									dCurrent += ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										ACTrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								case LatticeDispersal::e_AA:		// Calculate the area-to-area transition probabilities
									dCurrent += AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() + double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() + double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams) +
										AATrans(
											pX1->GetElement(pDims[0]).GetAsDouble(),
											pX2->GetElement(pDims[0]).GetAsDouble(),
											pY1->GetElement(pDims[0]).GetAsDouble(),
											pY2->GetElement(pDims[0]).GetAsDouble(),
											pX1->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pX2->GetElement(pDims[1]).GetAsDouble() - double(iIter) * dArenaWidth,
											pY1->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											pY2->GetElement(pDims[1]).GetAsDouble() - double(iSubIter) * dArenaHeight,
											cWrap, cExtraParams);
									break;
								default:		// Raise exception if unknown approximation method selected
									StdExtensions::RaiseException("unknown approximation method selected", __LINE__, "GaussianLatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
								}
							}
							// Set the current element
							cOut.GetElement(pDims).Initialise(dCurrent);
							dRunTot += dCurrent;
						}
					}
					// Final normalisation
					for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
					{
						cOut.GetElement(pDims).Initialise(
							cOut.GetElement(pDims).Divide(dRunTot));
					}
					// Set the absorbing state to zero
					cOut.GetElement(pDims).Initialise(0.0);
					// ------ END OF PERIODIC BOUNDARY CORRECTION ------
					break;
				default:
					// Raise exception if unknown boundary conditions selected
					StdExtensions::RaiseException("unknown boundary condition selected", __LINE__, "LatticeTransition(SEXP)", __FILE__, StdExtensions::e_DomainError);
				}
			}
			// Set the transition probabilities for the absorbing state
			for(pDims[1] = 0; pDims[1] < iNumCells; ++pDims[1])
			{
				cOut.GetElement(pDims).Initialise(0.0);
			}
			cOut.GetElement(pDims).Initialise(1.0);
			// Export the transition matrix
			pOut = cOut.ExportAsSEXP();
		}
	}
	catch(StdExtensions::Exception cXa)
	{	// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(std::bad_cast)
	{	// Write error if casting exception is thrown
		Rf_error("error thrown during pointer casting process");
	}
	catch(...)
	{	// Write error if a non-standard exception is thrown
		Rf_error("unknown error");
	}
	return pOut;
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
