#ifndef __RANDOM_H__
#define __RANDOM_H__

#include "MathFuncs.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{	// -------------------------------
	//  RANDOM NUMBER GENERATOR CLASS
	// -------------------------------
	// Abstract base class for generating random numbers.  Uses uniform
	// virtual function to generate random numbers from different distributions
	class Random
	{
	private:
		bool bNormEx;			// Boolean denoting whether there is an extra normal deviate to use
		double dNormEx;			// Extra value generated as part of Box-Muller transform
	protected:
		// Generate a number from a uniform distribution between 0 and 1
		virtual double UniformGen() throw() = 0;
	public:
		// Null initialisation functions
		Random() throw() : bNormEx(false), dNormEx(0.0) {};
		void Initialise() throw();
		virtual ~Random() throw() {Initialise();};
		// Seed the random number generator - pure virtual function
		virtual void Seed(int iSeedVal) throw() = 0;
		// Generate a uniform random number between dA and dB
		double Uniform(double dA = 0.0, double dB = 1.0) throw();
		// Generate a discrete uniform random number between dA and dB
		int DiscreteUniform(int iA, int iB) throw();
		// Generate a normal random number with mean dMean and standard deviation dSD
		double Normal(double dMean = 0.0, double dSD = 1.0) throw(StdExtensions::Exception);
	};
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
